// BlendWeight.cpp : Defines the entry point for the DLL application.
//

/*
	Blendweight plugin for AviSynth

	Copyright (C) 2006 Mohan
  
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation.
	 
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
		
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	
	The author can be contacted at the doom9.org forum under mg262.
*/

#include "stdafx.h"

#include "windows.h"
#include "avisynth.h"

#include <iostream>
#include <iomanip>
#include <fstream>
#include <cmath>

using namespace std;

/*
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    return TRUE;
}
*/


class BlendWeight : public GenericVideoFilter {    	

	double storeWeight;

	ofstream log;
public:

	BlendWeight(PClip _child, int startFrame, int endFrame, IScriptEnvironment* env);
	~BlendWeight();	
	
	double calcWeight(int frameNo, IScriptEnvironment* env);
};

BlendWeight::BlendWeight(PClip _child, int startFrame, int endFrame, IScriptEnvironment* env) :
	GenericVideoFilter(_child), log("BlendWeight log.txt")
{
	if (!vi.IsYV12())		env->ThrowError("BlendWeight requires YV12");   
//	if (frameNo == 0)		env->ThrowError("BlendWeight: blended frame has no predecessor");   
//	if (frameNo == )		env->ThrowError("BlendWeight: blended frame has no successor");   

	for(int i = startFrame; i <= endFrame; ++i)// i+=2)
        double weight = calcWeight(i, env);

	storeWeight = weight;
//	{ofstream log}
	/*
	ofstream output("weight.txt");
	output << weight;

	ofstream log("BlendWeight log.txt", ios_base::app);
	log.setf(ios_base::fixed, ios_base::doublefield);
	log.precision(4);
	log << "Frame " << frameNo << " = " << 1-weight << " x prev. Frame + " << weight << " x next Frame\n";
*/
//	ofstream phase("phase log.txt", ios_base::app);

/*	const double oldFPS = 23.976;
	const double newFPS = 25;
	const double ratio = oldFPS/newFPS*/

//	phase << 10000.0 - (23.976/25.0)* weight - (23.976/25.0)* frameNo << "\n";
	log.flush();
}

double BlendWeight::calcWeight(int frameNo, IScriptEnvironment* env)
{
	PVideoFrame blend = child->GetFrame(frameNo, env);
	//PVideoFrame prev  = child->GetFrame(frameNo-2, env);
	//PVideoFrame next  = child->GetFrame(frameNo+2, env);
	PVideoFrame prev  = child->GetFrame(frameNo-1, env);
	PVideoFrame next  = child->GetFrame(frameNo+1, env);
	const unsigned char* blendp = blend->GetReadPtr();
	const unsigned char* prevp  = prev->GetReadPtr();
	const unsigned char* nextp  = next->GetReadPtr();

	const int blendpitch  = blend->GetPitch();
	const int prevpitch  = next->GetPitch();
	const int nextpitch  = prev->GetPitch();
	const int width  = blend->GetRowSize();
	const int height = blend->GetHeight();

	//lambda = (x-p).(n-p) / (n-p).(n-p) 

	double num = 0.0, denom = 0.0;
	double origNorm = 0.0;

	for(int y = 0; y < height; ++y)
	{
		int rowNum = 0, rowDenom = 0;
		int rowOrigNorm = 0;

		for(int x = 0; x < width; ++x)
		{
			int prevVal = *(prevp + x);
			int blendVal = *(blendp + x);
			int nextVal = *(nextp + x);

			rowNum		+= (blendVal - prevVal) * (nextVal - prevVal);
			rowDenom	+= ( nextVal - prevVal) * (nextVal - prevVal);
			rowOrigNorm += (blendVal - prevVal) * (blendVal - prevVal);
		}
		blendp += blendpitch;
		prevp += prevpitch;
		nextp += nextpitch;

		num += double(rowNum);
		denom += double(rowDenom);
		origNorm += double(rowOrigNorm);
	}

	double xp2 = origNorm/(height*width);
	double np2 = denom/(height*width);
	double xpnp = num/(height*width);
	double xn2 = xp2 - 2*xpnp + np2;


	double weight = num/denom;
	double match = (origNorm - (2 * weight * num) + (weight * weight * denom) );

//	ofstream log("BlendWeight log.txt", ios_base::app);
	log.setf(ios_base::fixed, ios_base::floatfield);
	//log.precision(2);
//	if(match / (denom + 0.01) < 0.06 && weight > 0.1 && weight < 0.9)
	{
/*
	const double  threshhold = 0.02;
	if( abs(weight - 0.2) < threshhold )
		log << "8: ";
	else if( abs(weight - 0.4) < threshhold )
		log << "6: ";
	else if( abs(weight - 0.6) < threshhold )
		log << "4: ";
	else if( abs(weight - 0.8) < threshhold )
		log << "2: ";
	else 
		log << "   ";

	*/


	log << setprecision(1) << double(frameNo)/2  << "\t" 
//		<< ": (\t" 
		<< setprecision(4) << weight  << "\t\t" 
//		<< "\t)" 
		<< setprecision(4) << xp2  << "\t" 
		<< setprecision(4) << xn2  << "\t" 
		<< setprecision(4) << np2  << "\t" 
		<< setprecision(4) << xpnp  << "\t" 
		
	//	<< ")\t" << match / 100000.0
//		<< setprecision(2) <<  "\t" << match / denom * 100
		<< "\n";

	//log << abs(weight - 0.2) << "\n";
	}

	return weight;
}

BlendWeight::~BlendWeight() {}


AVSValue __cdecl Create_BlendWeight(AVSValue args, void* user_data, IScriptEnvironment* env) {
    return new BlendWeight(
		args[0].AsClip(),
		args[1].AsInt(0),
		args[2].AsInt(0),
		env);      
}


extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env) {
    env->AddFunction("BlendWeight", "cii", Create_BlendWeight, 0);

	return "BlendWeight plugin";
}









